; OKBASIC handles the basic IO files for bload
; there are two types of files, interlaced and non interlaced.
; the non interlaced are handled by the read nad write_raw routines
; the interlace routines are here
;
DSEG SEGMENT PUBLIC 'DATA'
assume ds:dseg,cs:cseg,es:dseg
extrn file_off:word,wr_switch:byte,handle:word
extrn width_bytes:word,height_pix:word,bas_header:word,bas_headero:word
extrn  vint:word,vphase:word,pass:word,buffer:word
extrn  hal_header:word,hal_headero:word
extrn hash_segment:word
include f:okfiles.asm
;
;  local data
;
vbuff     dw  0
vlace     dw  0
widv      dw  0
lenv      dw  0
buffdone  dw  0
hal_head_size dw 0
savedi    dw  0   ; location in di of interlace
savebp    dw  0   ; count of bytes
savecx    dw  0
phase     dw  0
vlines    dw  0
spanv     dw  0
;
DSEG ENDS
;
CSEG SEGMENT PUBLIC 'CODE'
assume ds:dseg,cs:cseg,es:dseg
extrn get_file:near,merge_file:near,reset_segs:near,read_raw:near
extrn write_raw:near,read_file:near,write_file:near,fill_buffer:near
extrn flush_buffer:near,cc_setup:near,next_wait:near
extrn calc_file_size:near
; read halo pic files. These are like compressed bload files
;
hal_read:
public hal_read
     call  reset_segs
     cmp  vlace,1
     jg    no_h1
     mov  vint,0
     call  hal_no_int
     ret
hal_no_int:
     stc
     ret
bad_hal_file:
     stc
     ret
no_h1:
; have to do the interlace bit
; first read it in to 0
     mov  cx,502
     sub  cx,hal_head_size
     mov  savedi,0
     mov  savebp,0
     mov  savecx,0
     mov  spanv,2000h
; calculate number of lines
     lea  dx,phase1
     mov  phase,dx
next_phase:
     call read_hal_block
     jnc   ok_hal_file
     jmp   end_hal_file
ok_hal_file:
     mov  es,hash_segment
     mov  ds,hash_segment
; cx has the number of bytes to work on
     mov  si,0
     mov  di,512
; uncompress the si into the di
hal_loop:
     cmp   cx,0
     jle   end_hal_loop
     lodsb
     dec  cx
     cmp  al,80h
     jne  by_eob
     jmp  end_hal_loop
by_eob:
     test al,80h
     jnz  dup_data
     push  cx
     mov   ah,0
     mov   bx,ax
     mov   cx,ax
     cld
rep  movsb
     pop  cx
     sub  cx,bx
     jmp  hal_loop
dup_data:
     push  cx
     and  ax,07fh
     mov  cx,ax
     lodsb
     cld
rep  stosb
     pop  cx
     dec  cx
     jmp  hal_loop
end_hal_loop:
     call  reset_segs
     mov   si,512
     mov   bp,di
     sub   bp,512
     cmp   bp,0
     jg    p1
     jmp   end_hal_file
p1:
     jmp   word ptr ds:phase
phase1:
; start phasing, for interlace
     mov   di,0
phase_ent:
; bp has the count of bytes to write to buffer
     mov   dx,widv
     shl   dx,1
     mov   bx,spanv
     mov   cx,bp
     push  ds
     push  es
     mov   es,buffer
     mov   ds,hash_segment
     cld
move_again:
     mov   cx,dx
     shr   cx,1
     cmp   cx,bp
     jg    move_hal
     jmp   last_hal_line
move_hal:
     push  di
rep   movsb
     pop   di
     mov   cx,dx
     shr   cx,1
     add   di,cx
     sub   bp,cx
     cmp    di,2000h
     jl     end_phase1
     mov   di,0
     add   di,bx
end_phase1:
     jmp    move_again
last_hal_line:
     mov   cx,bp
     cmp   bp,0
     jg    move_again
     pop   es
     pop   ds
; moved in, perhaps part of the move belongs to interlace
     mov   savedi,di
     lea   dx,phase2
     mov   phase,dx
     mov   cx,512
     jmp   next_phase
phase2:
; in this one we set di to be the end of next block
;
     mov   di,savedi
     jmp   phase_ent
end_hal_file:
     call  reset_segs
     mov   ax,lenv
     mov   cx,widv
     mul   cx
     mov   cx,ax
     push  ds
     mov   ds,buffer
     call  flush_buffer
     pop   ds
     clc
     ret
read_hal_block:
; this reads a 512 byte block into the hash_segment
; cx has the amount to read
     push  di
     mov  dx,0
     push  ds
     mov   ds,hash_segment
     call read_file
     pop   ds
     pop   di
     cmp   ax,0
     je    bad_hal_block
     mov   cx,ax
     clc
     ret
bad_hal_block:
     stc
     ret
;
;
; there are 7 or so different kinds of basic files, they are
; set up through the following entries
      nop   ; this is weird, but the basic stuff got loaded first
            ; and had an address of 0 so it would not execute
setf  macro cnt,buff,lace,wid,len
extrn  file_type:byte
get_bas&cnt:
      call reset_segs
      mov  cs:file_type[85],"B"
      mov  cs:file_type[86],"A"
      mov  cs:file_type[87],"S"
      mov  file_off,85
      mov  wr_switch,0
      mov  vbuff,buff
      mov  vlace,lace
      mov  widv,wid
      mov  lenv,len
      jmp  get_file
merge_bas&cnt:
      call reset_segs
      mov  cs:file_type[85],"B"
      mov  cs:file_type[86],"A"
      mov  cs:file_type[87],"S"
      mov  file_off,85
      mov  wr_switch,0
      mov  vbuff,buff
      mov  vlace,lace
      mov  widv,wid
      mov  lenv,len
      jmp  merge_file
save_bas&cnt:
      call reset_segs
      mov  cs:file_type[85],"B"
      mov  cs:file_type[86],"A"
      mov  cs:file_type[87],"S"
      mov  file_off,85
      mov  wr_switch,1
      mov  vbuff,buff
      mov  vlace,lace
      mov  widv,wid
      mov  lenv,len
      jmp  merge_file
public get_bas&cnt,merge_bas&cnt,save_bas&cnt
      endm
;
setf   1,0b800h,2,80,200   ; cga
setf   2,0a000h,1,80,350   ; ega
setf   3,0b800h,4,80,400   ; att
setf   4,0b000h,4,90,348   ; herc
setf   5,0a000h,1,80,400   ; vga
setf   6,0a000h,1,80,480   ; xega
setf   7,0a000h,1,90,400   ; sega
setf   8,0a000h,1,100,600  ; 800*600 vga
setf   9,0a000h,1,64,512   ; 512*512 vga
get_PCP:
      public get_pcp
      call reset_segs
      mov  cs:file_type[85],"P"
      mov  cs:file_type[86],"I"
      mov  cs:file_type[87],"C"
      mov  file_off,85
      mov  wr_switch,0
      mov  vbuff,0b800h
      mov  vlace,2
      mov  widv,80
      mov  lenv,200
      jmp  get_file
merge_PCP:
      public merge_pcp
      call reset_segs
      mov  cs:file_type[85],"P"
      mov  cs:file_type[86],"I"
      mov  cs:file_type[87],"C"
      mov  file_off,85
      mov  wr_switch,0
      mov  vbuff,0b800h
      mov  vlace,2
      mov  widv,80
      mov  lenv,200
      jmp  merge_file
save_PCP:
      public save_pcp
      call reset_segs
      mov  cs:file_type[85],"P"
      mov  cs:file_type[86],"I"
      mov  cs:file_type[87],"C"
      mov  file_off,85
      mov  wr_switch,1
      mov  vbuff,0b800h
      mov  vlace,2
      mov  widv,80
      mov  lenv,200
      jmp  merge_file
;
; pc PAINT+ read stuff
;
hal_in:
public  hal_in
; get the hal header stuff
     call  reset_segs
     cmp   word ptr hal_header.hal_sig,'HA'
     jne    bad_hal_head
     mov    al,hal_header.hal_id
     cmp    al,0
     jle    bad_hal_head
     cmp    al,2
     jle    hal_cga
     cmp    al,7
     je     hal_herc
     cmp    al,21
     je     hal_ega
     cmp    al,71
     je     hal_vga
bad_hal_head:
     stc
     ret
hal_cga:
; read past the next 4 bytes
      lea  dx,bas_header
     mov   bp,6
      mov  cx,6
      call read_file
      jnc  good_cga_read
      ret
good_cga_read:
      mov  wr_switch,0
      mov  vbuff,0b800h
      mov  vlace,2
      mov  widv,80
      mov  lenv,200
      jmp   hal_setup
hal_herc:
      mov  bp,0
      mov  wr_switch,0
      mov  vbuff,0b000h
      mov  vlace,4
      mov  widv,90
      mov  lenv,348
      jmp   hal_setup
hal_ega:
hal_vga:
      mov  wr_switch,0
      mov  vbuff,0a000h
      mov  vlace,1
      mov  widv,80
      mov  lenv,350
      lea  dx,bas_header
      mov  cx,2
      call read_file
      jnc  good_ega_read
bad_ega_head:
      stc
      ret
good_ega_read:
; bas_head has the mode
     mov   bp,2
     mov   ax,bas_header
     cmp   al,0
     jl    bad_ega_head
     cmp   al,1
     jle   good_cga_read
     cmp   al,3
     je    ega_3
     cmp   al,4
     je    ega_4
     cmp   al,10
     je    ega_4
     cmp   al,11
     je    ega_11
     cmp   al,12
     je    ega_11
     stc
     ret
ega_3:
      mov  lenv,200
      jmp   hal_setup
ega_4:
      mov  lenv,350
      jmp   hal_setup
ega_11:
      mov  lenv,400
      jmp   hal_setup
hal_setup:
     mov   hal_head_size,bp
     mov   ax,widv
     mov   width_bytes,ax
     mov   ax,lenv
     mov   height_pix,ax
     clc
     ret
bas_in  proc  near
public  bas_in
; get the basic header stuff
     call  reset_segs
     mov   ax,vbuff
     cmp   bas_header+3,ax
     je    ok_bas_head
     stc
     ret
ok_bas_head:
pcp_in:
public  pcp_in
     mov   ax,widv
     mov   width_bytes,ax
     mov   ax,lenv
     mov   height_pix,ax
     clc
     ret
bas_in  endp
;
bas_out proc  near
pcp_out:
public  bas_out
public  pcp_out
; write the bas header
     call  reset_segs
     call  calc_file_size
     mov   ax,vbuff
     mov   bas_headero+3,ax
     mov   cx,widv
     mov   width_bytes,cx
     mov   ax,lenv
     mov   height_pix,ax
     mul   cx
     mov   bas_headero+7,ax
     mov   cx,bas_headero
     lea   dx,bas_headero+2
     call  write_file
     cmp   vlace,1
     jg    init_bas_file
     clc
     ret
init_bas_file:
; interlaced files have to be initialized to binary zero
; first clean out the buffer
     mov   bx,buffer
     mov   cx,4000h
     mov   di,0
     mov   ax,0
     cld
     mov   es,bx
rep   stosw
     call  reset_segs
; now figure out how big he is
     mov   ax,2000h
     mul   vlace
     mov   cx,ax
     mov   dx,0
     mov   bx,buffer
     mov   ds,bx
     call  write_file
     call  reset_segs
; now reposition the file back to beginning
     mov   bx,handle
     mov   dx,bas_headero
     mov   cx,0
     mov   ah,42h  ; lseek
     mov   al,0
     int   21h
     ret
bas_out endp
;
bas_read proc near
pcp_read:
public  pcp_read
public  bas_read
; get the basic file
     call  reset_segs
     cmp  vlace,1
     jg    no_r1
     mov  vint,0
     call  read_raw
     ret
no_r1:
; basic read interlaced
; first read in a buffer full of stuff, an interlace at a time
     mov  cx,vlace
     mov  ax,width_bytes
     mov  dx,0
     mul  height_pix
     div  cx
     mov  bp,ax  ; buffersize
     mov  dx,576
     push ds
     push es
     mov  ax,buffer
     mov  ds,ax
     mov  es,ax
; cx has the number of segments
;
rbase_loop:
     push cx
     push dx
     push bp
     mov  cx,2000h
     call read_file
     pop  bp
     pop  dx
     pop  cx
     cmp  ax,0
     je   by_rb
     add  dx,bp
     loop rbase_loop
by_rb:
     pop  es
     pop  ds
; the buffer has a long contiguous count of lines
; starting at 756
;
;    now figure lines for each segmentt
;
     mov  ax,height_pix
     mov  dx,0
     mul  width_bytes
     div  vlace
     mov  bx,ax     ; bx has the length of a buffer
     mov  ax,height_pix
     mov  dx,0
     mov  cx,vlace
;    div  cx
     mov  cx,ax    ; cx has the count of lines
     mov  bp,vlace ; bp will have the interlace count
     mov  dx,width_bytes
     mov  di,0
     mov  si,576
     mov  ax,buffer
     mov  es,ax
     mov  ds,ax
sort_bas_loop:
     push cx
     push si
     mov  cx,bp      ; vlace is in bp
sb_loop1:
     push cx
     push si
; si points to line in segment
     mov  cx,dx  ; length of line
     cld
rep  movsb
;
     pop  si
     pop  cx
     add  si,bx
     loop  sb_loop1
;
     pop  si
     pop  cx
; have finished 2-4 lines of interlace
;
;    check to see if there is a buffer to flush
; in other words, are we within 360 bytes of the next buffer
     push  di
     add   di,361
     cmp   si,di
     pop   di
     jg    ok_bas_si  ; haven't caught up yet
; flush the buffer here
     cmp   di,0
    je  ok_bas_si
do_r_flush:
     push  bx
     push  cx
     push  dx
     push  bp
     push  si
     push  di
     push  ds
     push  es
;
     mov   cx,di
     call  flush_buffer
     pop   ds
     pop   es
     pop   di
     pop   si
     pop   bp
     pop   dx
     pop   cx
     pop   bx
     mov   di,0
;
ok_bas_si:
     add   si,dx
     loop  sort_bas_loop
     cmp   di,0
     jne   end_r_bas
     mov   cx,di
     call  flush_buffer
end_r_bas:
     call  reset_segs
     stc
     ret
bas_read endp
;
; this is the basic write routine. Non-interlaced modes just
; use write_raw, interlaced must play with the file
;
bas_write proc near
pcp_write:
public  pcp_write
public  bas_write
; write out the basic file
     call  reset_segs
     cmp  vlace,1
     jg    no_r2
     mov  vint,0
     call  write_raw
     ret
no_r2:
; This is interlaced. Vlace has the number of interlaced segments
; the segments are read sequentially from the video segments
; and then must be positioned in the file, as far as I know
; there is no good way to do this except move the r/w pointer
;
; prime the pump by filling up a buffer
;
     call  reset_segs
     mov   pass,0
     call  cc_setup
     call  reset_segs
     call  fill_buffer  ; get a bufferlead of stuff
; the amount in the buffer returns in ax
     mov   bp,ax  ; we'll save the amount in the buffer because we will
                  ; be subtracting from it
     call  reset_segs
; need to know how many lines in each interlaced segment
     mov   ax,height_pix
     mov   dx,0
     div   vlace
     mov   cx,ax
;  cx now has the amount of lines in each pass
     mov   si,0   ; position in buffer
; to recap,
;   bp has the count of bytes in the buffer
;   cx has the amount of lines for each interlaced segment
;   si has the position in the buffer
;
basw_loop:
     push   cx
     call   iwrite_bas  ; a routine to write out a interlace
     pop    cx
     jnc    write_2
     stc
     ret
write_2:
     loop   basw_loop
     stc
     ret
;
;
iwrite_bas:
; this writes out interlaced lines by positioning the w/r pointer
; it does the interlace up to 4 times
; get the current r/w/ pointer by moving relatively 0 bytes
     mov    dx,0
     mov    cx,0
     mov    ah,42h  ; lseek
     mov    bx,handle
     mov    al,1    ; releative to current position in file
     int    21h
;  dx:ax has the current position
     push   ax
     push   dx
     call   reset_segs
     mov    cx,vlace
     jmp    iw_loop2
iw_loop:
; this is the loop for cx times through a screen
     call  reset_segs
     push   cx
; position the read_write pointer 2000h ahead
     push   bp
     mov    dx,2000h
     sub    dx,width_bytes
     mov    bx,handle
     mov    cx,0
     mov    ah,42h  ; lseek
     mov    al,1
     int    21h
     pop    bp
     pop    cx
iw_loop2:  ; for first time through don't position pointer
     push   cx
     push   bp
     push   si
     push   ds
     mov    ax,buffer
     mov    dx,si
     mov    cx,width_bytes
     mov    ds,ax
     call   write_file
     pop    ds
     pop    si
     pop    bp
     add    si,width_bytes  ; next line
     mov    bx,bp
     dec    bx
     cmp    si,bx
     jl     by_new_fill
; here the current buffer is run out so we have to get some more
     call   next_wait
     call   fill_buffer
     jnc    good_ifill
; there is nothing left
; pop off the rest of the stack and return
     pop    cx
; this is th old buffer position
     pop    cx
     pop    dx
     stc
     ret
good_ifill:
     mov    bp,ax  ; new amount in buffer
     mov    si,0
by_new_fill:
; we go back to write another record
      pop   cx
     loop   iw_loop
; now reposition back to the next point in file
     pop    cx
     pop    dx
     add    dx,width_bytes
     mov    ah,42h  ; lseek
     mov    bx,handle
     mov    al,0    ; absolute
     int    21h
;  dx:ax has the current position
     clc
     ret
; end of the basic write routine
;
bas_write  endp
CSEG ENDS
     END
;
