; OKSTART is the program which begins and ends OPTIKS.
; OKSTART performs the following functions on start up
;
; 1) Saves entry state
; 2) sets up segment registers
; 3) sets up buffers for i/o
; 4) reads the config.ok file
; 5) reserves memory for screen buffer
; 6) allocates disk space for disk buffer
; 7) resets ctrl-break to the main entry program
; 8) branches to main control
;
; on exit OKSTART
;
;  1) resets the ctrl-break
;  2) closes all open files
;  3) resets the video to entry
;  4) issues a thank you message
;  5) exits gracefully
;
;  PROCEDURES IN OKSTART
;
;   ENTRY - main line
;   reset_segs - resets segments to dseg
;   end_prog  - resets all and ends program
;
;
segmentsize EQU   8000h
lseg segment  public 'DATA'
            ASSUME DS:lseg,ES:lseg
extrn font0:byte
extrn font1:byte
lseg  ends
dseg        SEGMENT PUBLIC 'DATA'
            ASSUME CS:cseg,DS:dseg,SS:sseg,ES:dseg
extrn  vwid:word
            EXTRN hash_segment:WORD
EXTRN       entry_mode:BYTE,err2:BYTE,buffer_segment:WORD
EXTRN       comline:BYTE,font:WORD,comsi:word,save_font:word
EXTRN       psp:WORD,start_stack:WORD,ret_sp:WORD
EXTRN       maxlines:WORD,default:BYTE
EXTRN       default_file:BYTE,sav_brk:WORD,breaker:WORD
extrn       mouse_on:byte,mode:byte,default_mode:byte
extrn       strg:byte,cdir:byte,cdisk:byte,sdisk:byte
extrn   curr_dta:byte,debug:byte,sdir:byte
extrn   thanks:byte,shareware:byte,curx:word,cury:word
extrn      ems_name:byte,ems_handle:word,ems_frame:word
extrn    ems_allocated:word,ems_free:word
extrn   ems_count:word
secs   db  0
;
dseg        ENDS
;
;
;
sseg        SEGMENT stack 'STACK'
            DW    64    DUP(0)
sseg        ENDS
cseg        SEGMENT PARA PUBLIC 'CODE'
PUBLIC      reset_segs,end_prog,exit_prog
            ASSUME CS:cseg,DS:dseg,SS:sseg,ES:dseg
EXTRN       read_file:NEAR,close_file:NEAR,open_in:NEAR
EXTRN       main:NEAR,set_screen:near,blop:near
extrn       erase_mem:near,init_finger:near,do_logo:near
extrn       ems_allocate:near,ems_dealloc:near
extrn       crit_err:near,mem_alloc:near
extrn       prbox:near
entry       PROC  far
; entry procedure
; shrink
; must shrink down to stack segment which is always last
            MOV   BX,sseg
            mov   ax,es
            sub   bx,ax
            add   BX,512      ;512*16 = 8192 bytes
            MOV   AH,4ah
            INT   21h
            jnc   ok_stack
            lea   dx,err2
            jmp   end_prog
ok_stack:
; set stack
            CLI
            MOV   SP,8190
            STI
; set up return
            XOR   AX,AX
            PUSH  DS
            PUSH  AX
            MOV   AX,dseg
            MOV   DS,AX
            MOV   psp,ES
            MOV   start_stack,SS
            MOV   ret_sp,SP
            MOV   AX,dseg
            MOV   ES,AX
; get current mode
            MOV   AX,0f00h    ; get mode
            INT   10h
            MOV   entry_mode,AL
;
; allocate memory areas
;
; check out the hash_seg
            MOV   AH,48h
            MOV   BX,0f00h   ; 20k +32K + 8k for herc bloads
            INT   21h
            jc    memory_error
            CMP   BX,0f00h
            JE    goodget2
memory_error:
            LEA   DX,err2
            JMP   end_prog
goodget2:
            MOV   hash_segment,AX
            add   ax,500h
            MOV   buffer_segment,AX
; get current dir
            call  get_cdir
; get configuration file
            CALL  get_default
;
; Allocate picture areas
            call  mem_alloc
            jc    memory_error
            call  reset_segs
            CALL  get_params
; set up the default font
            call  reset_segs
      assume ds:lseg
            lea   ax,font0
      assume  ds:dseg
            mov  save_font,ax
            MOV   font,AX
; set the DTA to curr_DTA
            lea   dx,curr_dta
            mov   ah,1ah
            int   21h
            cmp   debug,1
            jne   no_debug
            int   3        ; debug break
no_debug:
            call  reset_segs
; check to see if the comsi points to a ?
            mov    si,comsi
            cmp    comsi,0
            je    no_q
look_q:
            lodsb
            cmp   al,20h
            jne   check_q
            inc   comsi
            jmp   look_q
check_q:
            cmp   al,0
            je    no_q
            cmp   al,'?'
            jne   no_q
            inc   comsi
            mov   default_mode,6
no_q:
            CALL  set_screen
; set up BREAK key exit
            call  reset_segs
            CALL  break_on
            call  reset_segs
             call init_finger ; set up the finger data
            call  reset_segs
            call  erase_mem
            call  reset_segs
            call    do_logo            ; dispaly logo
            call  reset_segs
; check to see if a mouse is present
;
; check to see if a mouse driver is installed
            mov   ax,0
            mov   bx,33h*4
            mov   es,ax
            cmp   word ptr es:[bx+2],0
            jne   got_33a
            jmp   no_mouse
got_33a:
             mov  ax,0
            mov   bx,0
            int   33h    ; hope we come back
            cmp   ax,0
            jg    no_mouse
            cmp   bx,2
            jl    no_mouse
            cmp   bx,3
            jg    no_mouse
            mov   mouse_on,1
no_mouse:
            call  reset_segs
            JMP   main  ; go to mainline
exit_prog:
; this is the normal menu exit
            lea   dx,thanks
end_prog:
; in order to end the program there must be a message in dx
            PUSH  DX    ; this has the ending message
            CALL  reset_segs
            CALL  break_off
            jmp   sec_3
            mov   curx,32
            mov   cury,12
            mov   dx,offset shareware
            call  prbox
            CALL  reset_segs
; wait a second thing
            mov   ah,2ch
            int   21h
            mov   cx,10
sec_loop:
            push  cx
sec_loop2:
            mov   secs,dh
            mov   ah,1
            int   16h
            jz   sec_0
            mov   ah,0
            int   16h
            cmp   ax,250bh
            jne   sec_0
            pop   cx
            jmp   sec_3
sec_0:
            mov   ah,2ch
            int   21h
            cmp   secs,dh
            je    sec_loop2
sec_bloop:
            pop   cx
            loop  sec_loop
            call  blop
sec_1:
            mov   ah,1
            int   16h
            jz   sec_2
            mov   ah,0
            int   16h
            jmp   sec_1
sec_2:
            mov   ah,0
            int   16h
sec_3:
            CALL  reset_segs
            call  ems_dealloc
            CALL  reset_segs
            MOV   AL,entry_mode
            MOV   AH,0
            INT   10h
; check to see if the genius was on
            POP   DX
            MOV   AH,9
            INT   21h
            MOV   SP,ret_sp
            RET
entry       ENDP
other  proc near
get_cdir:
     call  reset_segs
     mov   cdir,'\'
     mov   sdir,'\'
; gets current dir and current disk
     mov   ah,19h
     int   21h
     add   al,41h
     mov   cdisk,al
     mov   sdisk,al
     lea   si,cdir+1
     mov   dx,0
     mov   ah,47h
     int   21h
     lea   si,sdir+1
     mov   dx,0
     mov   ah,47h
     int   21h
     ret
get_default:
; open in opens the file in STR!
            call reset_segs
            LEA   si,default_file
            lea   di,strg
            mov   cx,20
            cld
rep         movsb
            CALL  open_in
            JC    no_default
            LEA   DX,default
            MOV   CX,1024
            CALL  read_file
            CALL  close_file
            ret
no_default:
            mov    vwid,256
            RET

get_params:
            PUSH  DS
            MOV   AX,psp
            MOV   DS,AX
            MOV   CL,DS:[80h]
            MOV   CH,0
            LEA   DI,comline
            MOV   SI,81h
            CMP   CX,0
            JE    end_comline
            lea   ax,comline
            mov   es:comsi,ax
com_l_loop:
            LODSB
            CMP   AL,13
            JE    end_comline
            STOSB
            LOOP  com_l_loop
end_comline:
            POP   DS
            RET

reset_segs:
            PUSH  AX
            MOV   AX,dseg
            MOV   DS,AX
            MOV   ES,AX
            POP   AX
            RET
other   endp

eoi         EQU   20h   ;END OF INTERRUPT
brkint      EQU   1bh   ;BREAK INTERRUPT
break_on    PROC  NEAR
; this sets the control break, also sets critical error
;
; crit_err first
;
            LEA   DX,CS:crit_err
            PUSH  DS
            MOV   AX,CS
            MOV   DS,AX
            MOV   AX,2524h
            INT   21h
            POP   DS
;
            PUSH  ES
            MOV   AX,351bh
            INT   21h
; es:bx returns with the interrupt
            MOV   sav_brk,BX  ;SAVE OFFSET
            MOV   sav_brk+2,ES      ;SAVE SEGMENT
            POP   ES
            LEA   DX,CS:break_it
            PUSH  DS
            MOV   AX,CS
            MOV   DS,AX
            MOV   AX,251bh
            INT   21h
            POP   DS
            RET
break_on    ENDP
break_off   PROC  NEAR
            MOV   DX,sav_brk  ;RESET VECTOR BACK TO OLD RTN
            MOV   AX,sav_brk+2
            PUSH  DS
            MOV   DS,AX
            MOV   AX,251bh
            INT   21h
            POP   DS
            RET
break_off   ENDP
break_it    PROC  NEAR
; break handler
            MOV   AL,eoi
            OUT   020h,AL
            CALL  reset_segs
            MOV   AX,sseg
            MOV   SS,AX
            MOV   SP,breaker
            STI
            CALL  blop
            RET
break_it    ENDP
cseg        ENDS
            END   entry
