cseg segment para public 'CODE'
assume cs:cseg;ds:cseg,es:cseg
; PCPA.8 by Bob Montgomery, Orlando, Florida 8-14-87
; Unpacks PC Paint Full Page pictures to the EGA screen.
; Assembly language version for A86 assembler, produces .COM file directly.
      org 100h
begin:
     jmp  Start               ;Skip data

; =======================================================================
Header:                       ;File header; 17 bytes
ID        dw   0              ;ID for PC Paint files
Hor       dw   0              ;Horizontal size
Vert      dw   0              ;Vertical size
Xoff      dw   0              ;X offset into pic
Yoff      dw   0              ;Y offset into pic
Bpp       db   0              ;Lo nib=bits/pixel, Hi nib=additional planes
Ident     db   0              ;Should be 0FFh for PC Paint 2.0+
Vmode     db   0              ;Video mode for pic; we always use EGA
Descr     dw   0              ;Descriptor for rest of pic
Bcount    dw   0              ;Number of bytes of info to follow
; -------------------------------------------------------------
Palette   db   17 dup(0)      ;Palette reg contents and overscan reg
Pblks     dw   0              ;# packed blocks
; Error messages
Msg1:     db   'Cant find that file',13,10,'$'
Msg2:     db   'Not a PC Paint file',13,10,'$'
; Derived variables
Handle    dw   0              ;File handle
Databyte  db   0              ;Data byte
Hbytes    dw   0              ;Pic width in bytes
Colmask   db   0              ;Color mask
Ppb       db   0              ;Pixels/byte
Oldcolor  db   0              ;Color of previous pixel
Stdpal0   db   0,3,5,7        ;Standard palette 0 colors
Stdpal1   db   0,2,4,6        ;Standard palette 1 colors
Stdpal2   db   0,3,4,7        ;Standard palette 2 colors
Stdpal3   db   0,59,61,63     ;Standard palette 3 colors
Stdpal4   db   0,58,60,62     ;Standard palette 4 colors
Stdpal5   db   0,59,60,63     ;Standard palette 5 colors
; ===========================================================================

Start: ; Parse filespec.
     mov  si,80h              ;Point to command line
     lodsb                    ;Get # chars
     mov  cl,al               ;to cl
B0:  lodsb                    ;Skip spaces
     cmp  al,' '              ;= space?
     jne  B1                  ;No
     loop B0                  ;Yes, skip them
B1:  mov  dx,si               ;Save for file open
     dec  dx                  ;Set for 1st char of string
B2:  lodsb                    ;Get the char
     cmp  al,13               ;Is a carriage return?
     je   B3                  ;Yes
     loop B2                  ;Keep going until cr
B3:  mov  byte ptr [si-1],0            ;Make asciiz string
; Open the file.
     mov  ax,3D00h            ;Open file for read only
     int  21h
     mov  Handle,ax           ;Save handle
     lea  dx,Msg1             ;Point to message 1
     jc   B5                  ;Print it if file not opened
; Read header.
B4:  mov  bx,ax               ;Handle in bx
     mov  cx,17               ;Header
     lea  dx,Header           ;Point to file data area
     call Readfile
; Check for PC Paint Full Page file.
     mov  si,dx               ;Point to file data buffer
     lodsw                    ;Get ID
     cmp  ax,1234h            ;PC Paint file?
     je   C0                  ;Yes
     lea  dx,Msg2             ;No, print message 2
B5:  mov  ah,9
     int  21h
     int  20h                 ;and quit

; Get palette.
C0:  mov  cx,Bcount           ;Get # extra bytes
     mov  dx,offset Palette   ;Put then here
     call Readfile            ;Read from file
; Get # packed blocks.
     mov  cx,2                ;2 bytes
     mov  dx,offset Pblks     ;Put it here
     call Readfile            ;Get # packed blocks

; Set for EGA graphics mode.
     mov  ax,10h              ;Set EGA mode
     int  10h
; Set palette. If 4 color picture, move 4 colors to palette regs.
     mov  bl,0                ;Start with palette reg 0
     mov  cx,Bcount
     cmp  cx,2                ;4 color pic?
     jne  Pal16               ;No
     mov  cx,4                ;Yes, move standard palette to palette regs
     mov  si,offset Stdpal0   ;Point to 1st palette set
     mov  al,Palette          ;Get palette set (0 to 5)
     mov  ah,0                ;to ax
     shl  ax,1                ;x4 since 4 bytes/palette set
     shl  ax,1
     add  si,ax               ;Point to proper palette set
     mov  di,offset Palette   ;Point to palette entries
     rep  movsb               ;Move the 4 bytes to 1st 4 palette entries
Pal16:    ;Load the 16 palette regs from file data
     mov  dx,offset Palette   ;Point to palette
C1:  mov  ah,10h              ;Load the palette set
     mov  al,2                ;thru BIOS
     int  10h
; Enable S/R on all 4 planes.
     mov  dx,3CEh             ;Enable S/R on all planes
     mov  al,1
     out  dx,al
     inc  dx
     mov  al,0Fh
     out  dx,al
; Get pic width in bytes
     mov  ax,Hor              ;Get width in pixels
     mov  bl,8                ;Get pixels/byte
     div  bl                  ;Divide
     mov  Hbytes,ax           ;Save it
; Make color mask from Bpp (Bits/pixel); has 1's in last Bpp bits.
     mov  cl,Bpp              ;Get Bpp
     dec  cl                  ;# shift to do
     mov  ax,0101h            ;Will shift ah and or with al
C2:  shl  ah,1                ;Shift ah left
     or   al,ah               ;Or with al
     loop C2                  ;Do cl times
     mov  Colmask,al          ;Save color mask
     mov  ax,8                ;Get pixels/byte
     mov  cl,Bpp
     div  cl
     mov  Ppb,al              ;Save it
; Point to lower left corner of display.
     mov  ax,0A000h + 349*5   ;Start of last line
     mov  es,ax               ;to es
     xor  di,di               ;di=0 ; start of line
     mov  bl,80h              ;Mask in bl

; Get the next block of data to the buffer.
Nxtblk: ;Do next block.
     lea  dx,Buffer           ;Point to data buffer at end of file
     mov  cx,5                ;5 bytes in block header
     call Readfile            ;Read 5 bytes from file
     mov  si,dx               ;Point to buffer with ds:si
     lodsw                    ;Get packed bytes this block
     mov  cx,ax               ;to cx
     lodsw                    ;Get unpacked byte this block-ignore it
     lodsb                    ;Get run marker for this block
     mov  bh,al               ;Save in bh
     sub  cx,5                ;Get remaining bytes in block
     call Readfile            ;Read remaining bytes to buffer
     mov  si,dx               ;Point to start of buffer with ds:si

; Unpack this block to the EGA screen.
A0:  ;Get 1st byte.
     mov  ah,0
     lodsb                    ;Get byte
     cmp  al,bh               ;Marker?
     je   A1                  ;Yes, get pixel count
     dec  si                  ;Set pointer back
     mov  ax,1                ;Pixel count=1
     jmp  short A2            ;Get data byte
A1:  ;Get run count.
     dec  cx
     lodsb                    ;Get run count
     dec  cx                  ;Adjust bytes left
     cmp  al,0                ;=0?
     jne  A2                  ;No, write ax pixels
     lodsw                    ;Yes, get long pixel count
     sub  cx,2                ;Adjust bytes left
A2:  ;Get the data byte
     push cx                  ;Save bytes left
     mov  cx,ax               ;Set run count
     lodsb                    ;Get data byte
     mov  Databyte,al         ;Save it
; Write single byte or run to EGA screen.
A3:  mov  al,Databyte         ;Get databyte
     mov  ah,al               ;Save in ah
     push cx                  ;Save run count
     mov  cl,Ppb              ;Get Pixels/byte
     mov  ch,0                ;in cx
A4:  push cx                  ;Save it
     mov  cl,Bpp              ;Get shift count
     rol  ah,cl               ;Get 1st pixel color in last Bpp bits
     mov  al,ah               ;to al
     and  al,Colmask          ;Get color in al
     pop  cx                  ;Get Pixels/byte
     call Dopixel             ;Write the pixel
     shr  bl,1                ;Shift mask right
     loop A4                  ;Do all pixels for this byte?
     pop  cx                  ;Get run count
     cmp  bl,0                ;Mask = 0?
     jne  A5                  ;No
     mov  bl,80h              ;Yes, reset mask
     inc  di                  ;Next EGA buffer byte
     cmp  di,Hbytes           ;Done with row?
     jb   A5                  ;No
     mov  ax,es               ;Yes, set segment for previous line
     sub  ax,5
     mov  es,ax
     xor  di,di               ;Reset di
A5:  loop A3                  ;Do all repeats
     pop  cx                  ;Get bytes left
     loop A0                  ;Do next run
     dec  Pblks               ;All blocks done?
     je   No_Nxtblk
     jmp  Nxtblk              ;No, do next block
No_Nxtblk:
; If all blocks are done, wait for a key and then quit.
     mov  ah,0                ;Wait for a key
     int  16h
     mov  ax,3                ;Set for text mode
     int  10h
     int  20h                 ;and quit

; Subroutines ===============================================================
Dopixel:  ;Write the pixel mask (bl) @ es:di in EGA buffer. Preserve all regs.
     push dx                  ;Save regs used
     push ax                  ;Save color data
     mov  dx,3CEh             ;Point to graphics addr port
     cmp  al,Oldcolor         ;Same color as last pixel?
     je   D1                  ;Yes, no need to reset S/R register
     mov  al,0                ;Select S/R register
     out  dx,al               ;Send to port
     inc  dx                  ;Point to S/R reg port
     pop  ax                  ;Get data
     push ax
     out  dx,al               ;Send color data
     dec  dx                  ;Select bit mask port
D1:  mov  Oldcolor,al         ;Save this color
     mov  al,8
     out  dx,al
     inc  dx                  ;Bit mask port
     mov  al,bl               ;Get mask
     out  dx,al               ;Send it
     mov  al,es:[di]          ;Latch mask
     stosb                    ;Write pixel
     dec  di                  ;Comp for inc of di by stosb
     pop  ax                  ;Restore regs
     pop  dx
     ret                      ;and ret

Readfile: ;Read cx bytes from the file to the buffer at ds:dx.
     push ax
     push bx
     mov  bx,Handle           ;Get file handle
     mov  ah,3Fh              ;Read the data
     int  21h
     pop  bx
     pop  ax
     ret                      ;and return

Buffer equ $                  ;Start of file buffer
cseg  ends
      end   begin
